/*
   Copyright (c) 2013 LinkedIn Corp.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.linkedin.r2.filter.compression;

import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.io.IOUtils;
import org.xerial.snappy.Snappy;

/**
 * Wrapper for snappy compression algorithm
 * @author erli
 */
public class SnappyCompressor implements Compressor
{
  private static final String HTTP_NAME = "snappy";

  @Override
  public String getContentEncodingName()
  {
    return HTTP_NAME;
  }

  @Override
  public byte[] inflate(InputStream data) throws CompressionException
  {
    try
    {
      byte[] temp = IOUtils.toByteArray(data);
      return Snappy.uncompress(temp);
    }
    catch (IOException e)
    {
      throw new CompressionException(CompressionConstants.DECODING_ERROR + getContentEncodingName(), e);
    }

  }

  @Override
  public byte[] deflate(InputStream data) throws CompressionException
  {
    try
    {
      byte[] temp = IOUtils.toByteArray(data);
      return Snappy.compress(temp);
    }
    catch (IOException e)
    {
      throw new CompressionException(CompressionConstants.DECODING_ERROR + getContentEncodingName(), e);
    }
  }
}
